﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Text;
using System.Globalization;
namespace Persistence {

  public interface IPrimitiveSerializer {
    Type Type { get; }
    object Serialize(object o);
    object DeSerialize(object o);
  }

  public class String2XMLSerializer : IPrimitiveSerializer {
    public Type Type { get { return typeof(string); } }
    public object Serialize(object o) {
      return "<![CDATA[" +
        ((string)o).Replace("]]>", "]]]]><![CDATA[>") +
        "]]>";
    }
    public object DeSerialize(object o) {
      StringBuilder sb = new StringBuilder();
      foreach (string s in ((string)o).Split(
        new[] { "<![CDATA[", "]]>" },
        StringSplitOptions.RemoveEmptyEntries)) {
        sb.Append(s);
      }
      return sb.ToString();
    }
  }

  public class Int2XMLSerializer : IPrimitiveSerializer {
    public Type Type { get { return typeof(int); } }
    public object Serialize(object o) {
      return ((int)o).ToString();
    }
    public object DeSerialize(object o) {
      return int.Parse((string)o);
    }
  }

  public class Double2XmlSerializer : IPrimitiveSerializer {    
    public Type Type { get { return typeof(double); } }        
    public object Serialize(object o) {
      return ((double)o).ToString("r", CultureInfo.InvariantCulture);      
    }
    public object DeSerialize(object o) {
      return double.Parse((string)o, CultureInfo.InvariantCulture);
    }
  }

  public class Boolean2XmlSerializer : IPrimitiveSerializer {    
    public Type Type { get { return typeof(bool); } }    
    public object Serialize(object o) {
      return ((bool)o).ToString();
    }
    public object DeSerialize(object o) {
      return bool.Parse((string)o);
    }
  }

  public class DateTime2XmlSerializer : IPrimitiveSerializer {    
    public Type Type { get { return typeof(DateTime); } }      
    public object Serialize(object o) {
      return ((DateTime)o).Ticks.ToString();      
    }
    public object DeSerialize(object o) {
      return new DateTime(long.Parse((string)o));
    }
  }

  public abstract class NumberArray2XmlSerializer : IPrimitiveSerializer {
    public abstract Type Type { get; }    
    protected virtual string Separator { get { return ";"; } }
    protected abstract string formatValue(object o);
    protected abstract object parseValue(string o);
    public object Serialize(object obj) {      
      Array a = (Array)obj;
      StringBuilder sb = new StringBuilder();
      sb.Append(a.Rank);      
      for (int i = 0; i < a.Rank; i++) {
        sb.Append(Separator);
        sb.Append(a.GetLength(i));
      }
      foreach (object o in a) {
        sb.Append(Separator);
        sb.Append(formatValue(o));
      }
      return sb.ToString();
    }
    public object DeSerialize(object o) {
      IEnumerator values =
        ((string) o)
        .Split(new[] {Separator},
        StringSplitOptions.RemoveEmptyEntries).GetEnumerator();
      values.MoveNext();
      int rank = int.Parse((string)values.Current);
      int[] lengths = new int[rank];
      for ( int i = 0; i<rank; i++ ) {
        values.MoveNext();
        lengths[i] = int.Parse((string)values.Current);
      }
      Array a = Array.CreateInstance(this.Type.GetElementType(), lengths);
      int[] positions = new int[rank];
      while ( values.MoveNext() ) {
        a.SetValue(parseValue((string)values.Current), positions);
        positions[0] += 1;
        for ( int i = 0; i<rank-1; i++) {
          if (positions[i] >= lengths[i]) {
            positions[i] = 0;
            positions[i + 1] += 1;
          } else {
            break;
          }
        }
      }
      return a;
    }
  }
  public class IntArray2XmlSerializer : NumberArray2XmlSerializer {
    public override Type Type { get { return typeof(int[]); } }
    protected override string formatValue(object o) { return o.ToString(); }
    protected override object parseValue(string o) { return int.Parse(o); }
  }
  public class Int2DArray2XmlSerializer : IntArray2XmlSerializer {
    public override Type Type { get { return typeof (int[,]); } }
  }
  public class Int3DArray2XmlSerializer : IntArray2XmlSerializer {
    public override Type Type { get { return typeof(int[,,]); } }
  }
  public class DoubleArray2XmlSerializer : NumberArray2XmlSerializer {
    public override Type Type { get { return typeof(double[]); } }
    protected override string formatValue(object o) { return ((double) o).ToString("r"); }
    protected override object parseValue(string o) { return double.Parse(o); }    
  }
  public class Double2DArray2XmlSerializer : DoubleArray2XmlSerializer {
    public override Type Type { get { return typeof(double[,]); } }    
  }
  public class Double3DArray2XmlSerializer : DoubleArray2XmlSerializer {
    public override Type Type { get { return typeof(double[,,]); } }
  }

  public abstract class NumberEnumeration2XmlSerializer : IPrimitiveSerializer {
    public abstract Type Type { get; }
    protected virtual string Separator { get { return ";"; } }
    protected abstract void Add(IEnumerable enumeration, object o);
    protected abstract object Instantiate();
    protected abstract string formatValue(object o);
    protected abstract object parseValue(string o);    
    public object Serialize(object o) {
      StringBuilder sb = new StringBuilder();
      foreach (var value in (IEnumerable) o) {        
        sb.Append(formatValue(value));
        sb.Append(Separator);
      }
      return sb.ToString();
    }  
    public object DeSerialize(object o) {
      IEnumerable enumeration = (IEnumerable) Instantiate();        
      string[] values = ((string) o).Split(new[] {Separator}, StringSplitOptions.RemoveEmptyEntries);
      foreach ( var value in values ) {
        Add(enumeration, parseValue(value));
      }
      return enumeration;
    }
  }
  public class IntList2XmlSerializer : NumberEnumeration2XmlSerializer {
    public override Type Type { get { return typeof(List<int>); } }
    protected override void Add(IEnumerable enumeration, object o) { ((List<int>)enumeration).Add((int)o); }
    protected override object Instantiate() { return new List<int>(); }
    protected override string formatValue(object o) { return o.ToString(); }
    protected override object parseValue(string o) { return int.Parse(o); }
  }  
}